// file: $USFS/class/image_analysis/v3.0/imaga_ent_01.cc
//

// isip include files
//
#include "image_analysis.h"

#include "image_analysis_constants.h"
#include <integral_constants.h>

// method: ent_test_cc
//
// arguments:
//  float_8& sbe_a: (output) the image's sbe value
//  char_1* image_fname_a: (input) the image's filename
//
// return: a logical_1 value indicating the status
//
// this method computes a histogram and compares it to the reference model.
//
logical_1 Image_analysis::ent_test_cc(float_8& sbe_a,
				      char_1* image_fname_a) {
  
  // initialize
  //
  if (ent_init_cc() == ISIP_FALSE) {
    error_handler_cc((char_1*)"ent_test_cc",
		     (char_1*)"error while initializing for the entropy");
    return ISIP_FALSE;
  }
  
  // compute the standard deviation
  //
  if (ent_compute_cc(image_fname_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"ent_test_cc",
		     (char_1*)"error while computing the entropy");
    return ISIP_FALSE;
  }

  // provide some debugging information
  //
  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::ent_compute_cc",
	    "ent_npxls", huf_r_npxls_d);
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::ent_compute_cc",
	    "huf_r_nimages_d", huf_r_nimages_d); 
    
    fprintf(stdout, "%s\n", "the values of the entropy are");
    
    fprintf(stdout, "rent = %f\tgent = %f\tbent = %f\t\n", huf_t_red_d,
	    huf_t_grn_d, huf_t_blu_d);
  }  
  
  // compute the sbe
  //
  if (ent_compare_cc(sbe_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"ent_test_cc",
  	     (char_1*)"error while computing the sbe");
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
