// file: $USFS/class/image_analysis/v3.0/imaga_sd_08.cc
//

// isip include files
//
#include "image_analysis.h"

#include "image_analysis_constants.h"
#include <integral_constants.h>

// method: sd_compare_cc
//
// arguments:
//  float_8& sbe_a: (output) sbe value
//
// return: a logical_1 value indicating status
//
// this method computes an sbe from two histograms.
//
logical_1 Image_analysis::sd_compare_cc(float_8& sbe_a) {
  
  // declare local variables
  //
  float_8 diff_r = (float_4)0.0;
  float_8 diff_g = (float_4)0.0;
  float_8 diff_b = (float_4)0.0;
  float_8 rms_scaled;
  
  // compare the data from the model file with the computed data
  //
  diff_r = sd_r_red_d - sd_t_red_d;
  diff_g = sd_r_grn_d - sd_t_grn_d;
  diff_b = sd_r_blu_d - sd_t_blu_d;
  
  // weight the contributions from each color
  //
  rms_scaled = huf_rwgt_d * diff_r + huf_gwgt_d * diff_g +
    huf_bwgt_d * diff_b;

  // compute the scaled sbe
  //
  sbe_a = sd_ascale_d * rms_scaled + sd_bscale_d;
  
  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout,"<%s> sbe = %f\n",
	    "Image_analysis::sd_compare_cc", sbe_a);
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
